package cz.muni.pa165.common_library.knot;

import cz.muni.pa165.common_library.dtos.CarComponentResponseDto;
import cz.muni.pa165.common_library.dtos.CarResponseDto;
import cz.muni.pa165.common_library.dtos.DriverResponseDto;
import cz.muni.pa165.common_library.dtos.RaceDto;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

/**
 * Class for retrieving data from other modules using RestTemplate.
 */
@Component
public class CommonDbGetter {

  private final RestTemplate client = new RestTemplate();

  @Value("${server.services.race.port}")
  private String racePort;
  @Value("${server.services.driver.port}")
  private String driverPort;
  @Value("${server.services.component.port}")
  private String componentPort;
  @Value("${server.services.car.port}")
  private String carPort;


  /**
   * Get a driver using RestTemplate client.
   *
   * @param id driver id
   * @return Driver object
   */
  public Optional<DriverResponseDto> getDriverFromDb(Long id) {
    try {
      return fetchDriver("localhost", id);
    } catch (Exception ignored) {
      try {
        return fetchDriver("driver", id);
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<DriverResponseDto> fetchDriver(String source, Long id) {
    String url = "http://" + source + ":" + driverPort + "/driver/get/id?id=" + id;
    var driver = client.getForEntity(url, DriverResponseDto.class).getBody();
    return Optional.ofNullable(driver);
  }

  /**
   * Calls endpoint of the Driver module and retrieves all the drivers in db.
   *
   * @return List of all drivers in database.
   */
  public Optional<List<DriverResponseDto>> getAllDriversFromDb() {
    try {
      return fetchAllDrivers("localhost");
    } catch (Exception ignored) {
      try {
        return fetchAllDrivers("driver");
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<List<DriverResponseDto>> fetchAllDrivers(String source) {
    String url = "http://" + source + ":" + driverPort + "/driver/get/all";
    var drivers = client.getForEntity(url, DriverResponseDto[].class).getBody();
    return Optional.of(Arrays.stream(drivers).toList());
  }

  /**
   * Get a component using RestTemplate client.
   *
   * @param id component id
   * @return Component object
   */
  public Optional<CarComponentResponseDto> getComponentFromDb(Long id) {
    try {
      return fetchComponent("localhost", id);
    } catch (Exception ignored) {
      try {
        return fetchComponent("component", id);
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<CarComponentResponseDto> fetchComponent(String source, Long id) {
    String url = "http://" + source + ":" + componentPort + "/component/id?componentId=" + id;
    var components = client.getForEntity(url, CarComponentResponseDto.class).getBody();
    return Optional.ofNullable(components);
  }

  /**
   * Calls endpoint of the CarComponent module and retrieves all the car components in db.
   *
   * @return List of all car components in database.
   */
  public Optional<List<CarComponentResponseDto>> getAllComponentsFromDb() {
    try {
      return fetchAllComponents("localhost");
    } catch (Exception ignored) {
      try {
        return fetchAllComponents("component");
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<List<CarComponentResponseDto>> fetchAllComponents(String source) {
    String url = "http://" + source + ":" + componentPort + "/component/";
    var components = client.getForEntity(url, CarComponentResponseDto[].class).getBody();
    return Optional.of(Arrays.stream(components).toList());
  }

  /**
   * Calls endpoint of the car module to retrieve all cars form the database.
   *
   * @return optionally list of all existing cars.
   */
  public Optional<List<RaceDto>> getAllRaces() {
    try {
      return fetchAllRaces("localhost");
    } catch (Exception ignored) {
      try {
        return fetchAllRaces("race");
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<List<RaceDto>> fetchAllRaces(String source) {
    String url = "http://" + source + ":" + racePort + "/race/";
    var races = client.getForEntity(url, RaceDto[].class).getBody();
    return Optional.of(Arrays.stream(races).toList());
  }

  /**
   * Calls car module for a car given by id.
   *
   * @param carId id of a car.
   * @return carDto.
   */
  public Optional<CarResponseDto> getCar(Long carId) {
    try {
      return fetchCar("localhost", carId);
    } catch (Exception ignored) {
      try {
        return fetchCar("car", carId);
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<CarResponseDto> fetchCar(String source, Long carId) {
    String url = "http://" + source + ":" + carPort + "/car/?carId=" + carId;
    var car = client.getForEntity(url, CarResponseDto.class).getBody();
    return Optional.ofNullable(car);
  }

  /**
   * Calls endpoint of the car module to retrieve all cars form the database.
   *
   * @return optionally list of all existing cars.
   */
  public Optional<List<CarResponseDto>> getAllCars() {
    try {
      return fetchAllCars("localhost");
    } catch (Exception ignored) {
      try {
        return fetchAllCars("car");
      } catch (Exception ignoredAgain) {
        return Optional.empty();
      }
    }
  }

  private Optional<List<CarResponseDto>> fetchAllCars(String source) {
    String url = "http://" + source + ":" + carPort + "/car/all";
    var cars = client.getForEntity(url, CarResponseDto[].class).getBody();
    return Optional.of(Arrays.stream(cars).toList());
  }
}
